@@ -1,12 +1,7 @@
-# coding=utf-8
-from __future__ import unicode_literals
-
 from locale import getlocale
 from logging import getLogger
 
 from cached_property import threaded_cached_property
-from future.utils import python_2_unicode_compatible
-from six import string_types
 
 from .autodiscover import discover
 from .configuration import Configuration
@@ -20,43 +15,71 @@ from .folders import Folder, AdminAuditLogs, ArchiveDeletedItems, ArchiveInbox,
     Directory, Drafts, Favorites, IMContactList, Inbox, Journal, JunkEmail, LocalFailures, MsgFolderRoot, MyContacts, \
     Notes, Outbox, PeopleConnect, PublicFoldersRoot, QuickContacts, RecipientCache, RecoverableItemsDeletions, \
     RecoverableItemsPurges, RecoverableItemsRoot, RecoverableItemsVersions, Root, SearchFolders, SentItems, \
-    ServerFailures, SyncIssues, Tasks, ToDoSearch, VoiceMail, BaseFolder
-from .items import Item, BulkCreateResult, HARD_DELETE, \
-    AUTO_RESOLVE, SEND_TO_NONE, SAVE_ONLY, SEND_AND_SAVE_COPY, SEND_ONLY, ALL_OCCURRENCIES, \
-    DELETE_TYPE_CHOICES, MESSAGE_DISPOSITION_CHOICES, CONFLICT_RESOLUTION_CHOICES, AFFECTED_TASK_OCCURRENCES_CHOICES, \
-    SEND_MEETING_INVITATIONS_CHOICES, SEND_MEETING_INVITATIONS_AND_CANCELLATIONS_CHOICES, \
-    SEND_MEETING_CANCELLATIONS_CHOICES, ID_ONLY
-from .properties import Mailbox, SendingAs, FolderId, DistinguishedFolderId
+    ServerFailures, SyncIssues, Tasks, ToDoSearch, VoiceMail
+from .items import HARD_DELETE, AUTO_RESOLVE, SEND_TO_NONE, SAVE_ONLY, ALL_OCCURRENCIES, ID_ONLY
+from .properties import Mailbox, SendingAs
+from .protocol import Protocol
 from .queryset import QuerySet
 from .services import ExportItems, UploadItems, GetItem, CreateItem, UpdateItem, DeleteItem, MoveItem, SendItem, \
-    CopyItem, GetUserOofSettings, SetUserOofSettings, GetMailTips, ArchiveItem, GetDelegate
-from .settings import OofSettings
+    CopyItem, GetUserOofSettings, SetUserOofSettings, GetMailTips, ArchiveItem, GetDelegate, MarkAsJunk, GetPersona
 from .util import get_domain, peek
 
 log = getLogger(__name__)
 
 
-@python_2_unicode_compatible
-class Account(object):
-    """Models an Exchange server user account. The primary key for an account is its PrimarySMTPAddress
-    """
+class Identity:
+    """Contains information that uniquely identifies an account. Currently only used for SOAP impersonation headers."""
+
+    def __init__(self, primary_smtp_address=None, smtp_address=None, upn=None, sid=None):
+        """
+
+        :param primary_smtp_address: The primary email address associated with the account (Default value = None)
+        :param smtp_address: The (non-)primary email address associated with the account (Default value = None)
+        :param upn: (Default value = None)
+        :param sid: (Default value = None)
+        :return:
+        """
+        self.primary_smtp_address = primary_smtp_address
+        self.smtp_address = smtp_address
+        self.upn = upn
+        self.sid = sid
+
+    def __eq__(self, other):
+        for k in self.__dict__:
+            if getattr(self, k) != getattr(other, k):
+                return False
+        return True
+
+    def __hash__(self):
+        return hash(repr(self))
+
+    def __repr__(self):
+        return self.__class__.__name__ + repr((self.primary_smtp_address, self.smtp_address, self.upn, self.sid))
+
+
+class Account:
+    """Models an Exchange server user account."""
+
     def __init__(self, primary_smtp_address, fullname=None, access_type=None, autodiscover=False, credentials=None,
                  config=None, locale=None, default_timezone=None):
         """
+
         :param primary_smtp_address: The primary email address associated with the account on the Exchange server
-        :param fullname: The full name of the account. Optional.
+        :param fullname: The full name of the account. Optional. (Default value = None)
         :param access_type: The access type granted to 'credentials' for this account. Valid options are 'delegate'
-        (default) and 'impersonation'.
+            and 'impersonation'. 'delegate' is default if 'credentials' is set. Otherwise, 'impersonation' is default.
         :param autodiscover: Whether to look up the EWS endpoint automatically using the autodiscover protocol.
-        :param credentials: A Credentials object containing valid credentials for this account.
+            (Default value = False)
+        :param credentials: A Credentials object containing valid credentials for this account. (Default value = None)
         :param config: A Configuration object containing EWS endpoint information. Required if autodiscover is disabled
+            (Default value = None)
         :param locale: The locale of the user, e.g. 'en_US'. Defaults to the locale of the host, if available.
         :param default_timezone: EWS may return some datetime values without timezone information. In this case, we will
-        assume values to be in the provided timezone. Defaults to the timezone of the host.
+            assume values to be in the provided timezone. Defaults to the timezone of the host.
+        :return:
         """
         if '@' not in primary_smtp_address:
             raise ValueError("primary_smtp_address %r is not an email address" % primary_smtp_address)
-        self.primary_smtp_address = primary_smtp_address
         self.fullname = fullname
         # Assume delegate access if individual credentials are provided. Else, assume service user with impersonation
         self.access_type = access_type or (DELEGATE if credentials else IMPERSONATION)
@@ -66,19 +89,23 @@ class Account(object):
             self.locale = locale or getlocale()[0] or None  # get_locale() might not be able to determine the locale
         except ValueError as e:
             # getlocale() may throw ValueError if it fails to parse the system locale
-            log.warning('Failed to get locale (%s)' % e)
+            log.warning('Failed to get locale (%s)', e)
             self.locale = None
-        if not isinstance(self.locale, string_types + (type(None),)):
+        if not isinstance(self.locale, (type(None), str)):
             raise ValueError("Expected 'locale' to be a string, got %r" % self.locale)
-        try:
-            self.default_timezone = default_timezone or EWSTimeZone.localzone()
-        except (ValueError, UnknownTimeZone) as e:
-            # There is no translation from local timezone name to Windows timezone name, or e failed to find the
-            # local timezone.
-            log.warning('%s. Fallback to UTC', e.args[0])
-            self.default_timezone = UTC
-        if not isinstance(self.default_timezone, EWSTimeZone):
-            raise ValueError("Expected 'default_timezone' to be an EWSTimeZone, got %r" % self.default_timezone)
+        if default_timezone:
+            try:
+                self.default_timezone = EWSTimeZone.from_timezone(default_timezone)
+            except TypeError:
+                raise ValueError("Expected 'default_timezone' to be an EWSTimeZone, got %r" % default_timezone)
+        else:
+            try:
+                self.default_timezone = EWSTimeZone.localzone()
+            except (ValueError, UnknownTimeZone) as e:
+                # There is no translation from local timezone name to Windows timezone name, or e failed to find the
+                # local timezone.
+                log.warning('%s. Fallback to UTC', e.args[0])
+                self.default_timezone = UTC
         if not isinstance(config, (Configuration, type(None))):
             raise ValueError("Expected 'config' to be a Configuration, got %r" % config)
         if autodiscover:
@@ -88,18 +115,28 @@ class Account(object):
                     credentials = config.credentials
             else:
                 retry_policy, auth_type = None, None
-            self.primary_smtp_address, self.protocol = discover(
-                email=self.primary_smtp_address, credentials=credentials, auth_type=auth_type, retry_policy=retry_policy
+            self.ad_response, self.protocol = discover(
+                email=primary_smtp_address, credentials=credentials, auth_type=auth_type, retry_policy=retry_policy
             )
+            primary_smtp_address = self.ad_response.autodiscover_smtp_address
         else:
             if not config:
                 raise AttributeError('non-autodiscover requires a config')
-            self.protocol = config.protocol
+            self.ad_response = None
+            self.protocol = Protocol(config=config)
+
+        # Other ways of identifying the account can be added later
+        self.identity = Identity(primary_smtp_address=primary_smtp_address)
+
         # We may need to override the default server version on a per-account basis because Microsoft may report one
         # server version up-front but delegate account requests to an older backend server.
         self.version = self.protocol.version
         log.debug('Added account: %s', self)
 
+    @property
+    def primary_smtp_address(self):
+        return self.identity.primary_smtp_address
+
     @threaded_cached_property
     def admin_audit_logs(self):
         return self.root.get_default_folder(AdminAuditLogs)
@@ -282,102 +319,76 @@ class Account(object):
         # supports the 'del self.oof_settings' syntax to invalidate the cache, but does not support custom setter
         # methods. Having a non-cached service call here goes against the assumption that properties are cheap, but the
         # alternative is to create get_oof_settings() and set_oof_settings(), and that's just too Java-ish for my taste.
-        return GetUserOofSettings(account=self).call(
+        return GetUserOofSettings(account=self).get(
             mailbox=Mailbox(email_address=self.primary_smtp_address),
         )
 
     @oof_settings.setter
     def oof_settings(self, value):
-        if not isinstance(value, OofSettings):
-            raise ValueError("'value' %r must be an OofSettings instance" % value)
-        SetUserOofSettings(account=self).call(
-            mailbox=Mailbox(email_address=self.primary_smtp_address),
+        SetUserOofSettings(account=self).get(
             oof_settings=value,
+            mailbox=Mailbox(email_address=self.primary_smtp_address),
         )
 
     def _consume_item_service(self, service_cls, items, chunk_size, kwargs):
-        # 'items' could be an unevaluated QuerySet, e.g. if we ended up here via `some_folder.filter(...).delete()`. In
-        # that case, we want to use its iterator. Otherwise, peek() will start a count() which is wasteful because we
-        # need the item IDs immediately afterwards. iterator() will only do the bare minimum.
         if isinstance(items, QuerySet):
-            items = items.iterator()
+            # We just want an iterator over the results
+            items = iter(items)
         is_empty, items = peek(items)
         if is_empty:
             # We accept generators, so it's not always convenient for caller to know up-front if 'ids' is empty. Allow
             # empty 'ids' and return early.
             return
         kwargs['items'] = items
-        for i in service_cls(account=self, chunk_size=chunk_size).call(**kwargs):
-            yield i
+        yield from service_cls(account=self, chunk_size=chunk_size).call(**kwargs)
 
     def export(self, items, chunk_size=None):
-        """Return export strings of the given items
+        """Return export strings of the given items.
 
         :param items: An iterable containing the Items we want to export
-        :param chunk_size: The number of items to send to the server in a single request
+        :param chunk_size: The number of items to send to the server in a single request (Default value = None)
 
-        :return A list of strings, the exported representation of the object
+        :return: A list of strings, the exported representation of the object
         """
         return list(
-            self._consume_item_service(service_cls=ExportItems, items=items, chunk_size=chunk_size, kwargs=dict())
+            self._consume_item_service(service_cls=ExportItems, items=items, chunk_size=chunk_size, kwargs={})
         )
 
     def upload(self, data, chunk_size=None):
-        """Adds objects retrieved from export into the given folders
+        """Upload objects retrieved from an export to the given folders.
 
-        :param data: An iterable of tuples containing the folder we want to upload the data to and the
-            string outputs of exports.
-        :param chunk_size: The number of items to send to the server in a single request
+        :param data: An iterable of tuples containing the folder we want to upload the data to and the string outputs of
+            exports.
+        :param chunk_size: The number of items to send to the server in a single request (Default value = None)
 
-        :return A list of tuples with the new ids and changekeys
+        :return: A list of tuples with the new ids and changekeys
 
-        Example:
-        account.upload([(account.inbox, "AABBCC..."),
-                        (account.inbox, "XXYYZZ..."),
-                        (account.calendar, "ABCXYZ...")])
-        -> [("idA", "changekey"), ("idB", "changekey"), ("idC", "changekey")]
+          Example:
+          account.upload([(account.inbox, "AABBCC..."),
+          (account.inbox, "XXYYZZ..."),
+          (account.calendar, "ABCXYZ...")])
+          -> [("idA", "changekey"), ("idB", "changekey"), ("idC", "changekey")]
         """
-        is_empty, data = peek(data)
-        if is_empty:
-            # We accept generators, so it's not always convenient for caller to know up-front if 'upload_data' is empty.
-            # Allow empty 'upload_data' and return early.
-            return []
-        return list(UploadItems(account=self, chunk_size=chunk_size).call(data=data))
+        return list(
+            self._consume_item_service(service_cls=UploadItems, items=data, chunk_size=chunk_size, kwargs={})
+        )
 
     def bulk_create(self, folder, items, message_disposition=SAVE_ONLY, send_meeting_invitations=SEND_TO_NONE,
                     chunk_size=None):
-        """Creates new items in 'folder'
+        """Create new items in 'folder'.
 
         :param folder: the folder to create the items in
         :param items: an iterable of Item objects
         :param message_disposition: only applicable to Message items. Possible values are specified in
-               MESSAGE_DISPOSITION_CHOICES
+            MESSAGE_DISPOSITION_CHOICES (Default value = SAVE_ONLY)
         :param send_meeting_invitations: only applicable to CalendarItem items. Possible values are specified in
-               SEND_MEETING_INVITATIONS_CHOICES
-        :param chunk_size: The number of items to send to the server in a single request
+            SEND_MEETING_INVITATIONS_CHOICES (Default value = SEND_TO_NONE)
+        :param chunk_size: The number of items to send to the server in a single request (Default value = None)
+
         :return: a list of either BulkCreateResult or exception instances in the same order as the input. The returned
-                 BulkCreateResult objects are normal Item objects except they only contain the 'id' and 'changekey'
-                 of the created item, and the 'id' of any attachments that were also created.
+          BulkCreateResult objects are normal Item objects except they only contain the 'id' and 'changekey'
+          of the created item, and the 'id' of any attachments that were also created.
         """
-        if message_disposition not in MESSAGE_DISPOSITION_CHOICES:
-            raise ValueError("'message_disposition' %s must be one of %s" % (
-                message_disposition, MESSAGE_DISPOSITION_CHOICES
-            ))
-        if send_meeting_invitations not in SEND_MEETING_INVITATIONS_CHOICES:
-            raise ValueError("'send_meeting_invitations' %s must be one of %s" % (
-                send_meeting_invitations, SEND_MEETING_INVITATIONS_CHOICES
-            ))
-        if folder is not None:
-            if not isinstance(folder, BaseFolder):
-                raise ValueError("'folder' %r must be a Folder instance" % folder)
-            if folder.account != self:
-                raise ValueError('"Folder must belong to this account')
-        if message_disposition == SAVE_ONLY and folder is None:
-            raise AttributeError("Folder must be supplied when in save-only mode")
-        if message_disposition == SEND_AND_SAVE_COPY and folder is None:
-            folder = self.sent  # 'Sent' is default EWS behaviour
-        if message_disposition == SEND_ONLY and folder is not None:
-            raise AttributeError("Folder must be None in send-ony mode")
         if isinstance(items, QuerySet):
             # bulk_create() on a queryset does not make sense because it returns items that have already been created
             raise ValueError('Cannot bulk create items from a QuerySet')
@@ -388,50 +399,30 @@ class Account(object):
             message_disposition,
             send_meeting_invitations,
         )
-        return list(
-            i if isinstance(i, Exception)
-            else BulkCreateResult.from_xml(elem=i, account=self)
-            for i in self._consume_item_service(service_cls=CreateItem, items=items, chunk_size=chunk_size, kwargs=dict(
-                folder=folder,
-                message_disposition=message_disposition,
-                send_meeting_invitations=send_meeting_invitations,
-            ))
-        )
+        return list(self._consume_item_service(service_cls=CreateItem, items=items, chunk_size=chunk_size, kwargs=dict(
+            folder=folder,
+            message_disposition=message_disposition,
+            send_meeting_invitations=send_meeting_invitations,
+        )))
 
     def bulk_update(self, items, conflict_resolution=AUTO_RESOLVE, message_disposition=SAVE_ONLY,
                     send_meeting_invitations_or_cancellations=SEND_TO_NONE, suppress_read_receipts=True,
                     chunk_size=None):
-        """
-        Bulk updates existing items
+        """Bulk update existing items.
 
         :param items: a list of (Item, fieldnames) tuples, where 'Item' is an Item object, and 'fieldnames' is a list
-                      containing the attributes on this Item object that we want to be updated.
+            containing the attributes on this Item object that we want to be updated.
         :param conflict_resolution: Possible values are specified in CONFLICT_RESOLUTION_CHOICES
+            (Default value = AUTO_RESOLVE)
         :param message_disposition: only applicable to Message items. Possible values are specified in
-               MESSAGE_DISPOSITION_CHOICES
+            MESSAGE_DISPOSITION_CHOICES (Default value = SAVE_ONLY)
         :param send_meeting_invitations_or_cancellations: only applicable to CalendarItem items. Possible values are
-               specified in SEND_MEETING_INVITATIONS_AND_CANCELLATIONS_CHOICES
-        :param suppress_read_receipts: nly supported from Exchange 2013. True or False
-        :param chunk_size: The number of items to send to the server in a single request
+            specified in SEND_MEETING_INVITATIONS_AND_CANCELLATIONS_CHOICES (Default value = SEND_TO_NONE)
+        :param suppress_read_receipts: nly supported from Exchange 2013. True or False (Default value = True)
+        :param chunk_size: The number of items to send to the server in a single request (Default value = None)
 
         :return: a list of either (id, changekey) tuples or exception instances, in the same order as the input
         """
-        if conflict_resolution not in CONFLICT_RESOLUTION_CHOICES:
-            raise ValueError("'conflict_resolution' %s must be one of %s" % (
-                conflict_resolution, CONFLICT_RESOLUTION_CHOICES
-            ))
-        if message_disposition not in MESSAGE_DISPOSITION_CHOICES:
-            raise ValueError("'message_disposition' %s must be one of %s" % (
-                message_disposition, MESSAGE_DISPOSITION_CHOICES
-            ))
-        if send_meeting_invitations_or_cancellations not in SEND_MEETING_INVITATIONS_AND_CANCELLATIONS_CHOICES:
-            raise ValueError("'send_meeting_invitations_or_cancellations' %s must be one of %s" % (
-                send_meeting_invitations_or_cancellations, SEND_MEETING_INVITATIONS_AND_CANCELLATIONS_CHOICES
-            ))
-        if suppress_read_receipts not in (True, False):
-            raise ValueError("'suppress_read_receipts' %s must be True or False" % suppress_read_receipts)
-        if message_disposition == SEND_ONLY:
-            raise ValueError('Cannot send-only existing objects. Use SendItem service instead')
         # bulk_update() on a queryset does not make sense because there would be no opportunity to alter the items. In
         # fact, it could be dangerous if the queryset contains an '.only()'. This would wipe out certain fields
         # entirely.
@@ -444,46 +435,29 @@ class Account(object):
             message_disposition,
             send_meeting_invitations_or_cancellations,
         )
-        return list(
-            i if isinstance(i, Exception) else Item.id_from_xml(i)
-            for i in self._consume_item_service(service_cls=UpdateItem, items=items, chunk_size=chunk_size, kwargs=dict(
-                conflict_resolution=conflict_resolution,
-                message_disposition=message_disposition,
-                send_meeting_invitations_or_cancellations=send_meeting_invitations_or_cancellations,
-                suppress_read_receipts=suppress_read_receipts,
-            ))
-        )
+        return list(self._consume_item_service(service_cls=UpdateItem, items=items, chunk_size=chunk_size, kwargs=dict(
+            conflict_resolution=conflict_resolution,
+            message_disposition=message_disposition,
+            send_meeting_invitations_or_cancellations=send_meeting_invitations_or_cancellations,
+            suppress_read_receipts=suppress_read_receipts,
+        )))
 
     def bulk_delete(self, ids, delete_type=HARD_DELETE, send_meeting_cancellations=SEND_TO_NONE,
                     affected_task_occurrences=ALL_OCCURRENCIES, suppress_read_receipts=True, chunk_size=None):
-        """
-        Bulk deletes items.
+        """Bulk delete items.
 
         :param ids: an iterable of either (id, changekey) tuples or Item objects.
         :param delete_type: the type of delete to perform. Possible values are specified in DELETE_TYPE_CHOICES
+            (Default value = HARD_DELETE)
         :param send_meeting_cancellations: only applicable to CalendarItem. Possible values are specified in
-               SEND_MEETING_CANCELLATIONS_CHOICES.
+            SEND_MEETING_CANCELLATIONS_CHOICES. (Default value = SEND_TO_NONE)
         :param affected_task_occurrences: only applicable for recurring Task items. Possible values are specified in
-               AFFECTED_TASK_OCCURRENCES_CHOICES.
-        :param suppress_read_receipts: only supported from Exchange 2013. True or False.
-        :param chunk_size: The number of items to send to the server in a single request
+            AFFECTED_TASK_OCCURRENCES_CHOICES. (Default value = ALL_OCCURRENCIES)
+        :param suppress_read_receipts: only supported from Exchange 2013. True or False. (Default value = True)
+        :param chunk_size: The number of items to send to the server in a single request (Default value = None)
 
         :return: a list of either True or exception instances, in the same order as the input
         """
-        if delete_type not in DELETE_TYPE_CHOICES:
-            raise ValueError("'delete_type' %s must be one of %s" % (
-                delete_type, DELETE_TYPE_CHOICES
-            ))
-        if send_meeting_cancellations not in SEND_MEETING_CANCELLATIONS_CHOICES:
-            raise ValueError("'send_meeting_cancellations' %s must be one of %s" % (
-                send_meeting_cancellations, SEND_MEETING_CANCELLATIONS_CHOICES
-            ))
-        if affected_task_occurrences not in AFFECTED_TASK_OCCURRENCES_CHOICES:
-            raise ValueError("'affected_task_occurrences' %s must be one of %s" % (
-                affected_task_occurrences, AFFECTED_TASK_OCCURRENCES_CHOICES
-            ))
-        if suppress_read_receipts not in (True, False):
-            raise ValueError("'suppress_read_receipts' %s must be True or False" % suppress_read_receipts)
         log.debug(
             'Deleting items for %s (delete_type: %s, send_meeting_invitations: %s, affected_task_occurences: %s)',
             self,
@@ -501,20 +475,19 @@ class Account(object):
         )
 
     def bulk_send(self, ids, save_copy=True, copy_to_folder=None, chunk_size=None):
-        """ Send existing draft messages. If requested, save a copy in 'copy_to_folder'
+        """Send existing draft messages. If requested, save a copy in 'copy_to_folder'.
 
         :param ids: an iterable of either (id, changekey) tuples or Item objects.
-        :param save_copy: If true, saves a copy of the message
+        :param save_copy: If true, saves a copy of the message (Default value = True)
         :param copy_to_folder: If requested, save a copy of the message in this folder. Default is the Sent folder
-        :param chunk_size: The number of items to send to the server in a single request
+        :param chunk_size: The number of items to send to the server in a single request (Default value = None)
+
         :return: Status for each send operation, in the same order as the input
         """
         if copy_to_folder and not save_copy:
             raise AttributeError("'save_copy' must be True when 'copy_to_folder' is set")
         if save_copy and not copy_to_folder:
             copy_to_folder = self.sent  # 'Sent' is default EWS behaviour
-        if copy_to_folder and not isinstance(copy_to_folder, BaseFolder):
-            raise ValueError("'copy_to_folder' %r must be a Folder instance" % copy_to_folder)
         return list(
             self._consume_item_service(service_cls=SendItem, items=ids, chunk_size=chunk_size, kwargs=dict(
                 saved_item_folder=copy_to_folder,
@@ -522,39 +495,31 @@ class Account(object):
         )
 
     def bulk_copy(self, ids, to_folder, chunk_size=None):
-        """ Copy items to another folder
+        """Copy items to another folder.
 
         :param ids: an iterable of either (id, changekey) tuples or Item objects.
         :param to_folder: The destination folder of the copy operation
-        :param chunk_size: The number of items to send to the server in a single request
+        :param chunk_size: The number of items to send to the server in a single request (Default value = None)
+
         :return: Status for each send operation, in the same order as the input
         """
-        if not isinstance(to_folder, BaseFolder):
-            raise ValueError("'to_folder' %r must be a Folder instance" % to_folder)
-        return list(
-            i if isinstance(i, Exception) else Item.id_from_xml(i)
-            for i in self._consume_item_service(service_cls=CopyItem, items=ids, chunk_size=chunk_size, kwargs=dict(
-                to_folder=to_folder,
-            ))
-        )
+        return list(self._consume_item_service(service_cls=CopyItem, items=ids, chunk_size=chunk_size, kwargs=dict(
+            to_folder=to_folder,
+        )))
 
     def bulk_move(self, ids, to_folder, chunk_size=None):
-        """Move items to another folder
+        """Move items to another folder.
 
         :param ids: an iterable of either (id, changekey) tuples or Item objects.
         :param to_folder: The destination folder of the copy operation
-        :param chunk_size: The number of items to send to the server in a single request
+        :param chunk_size: The number of items to send to the server in a single request (Default value = None)
+
         :return: The new IDs of the moved items, in the same order as the input. If 'to_folder' is a public folder or a
-        folder in a different mailbox, an empty list is returned.
+          folder in a different mailbox, an empty list is returned.
         """
-        if not isinstance(to_folder, BaseFolder):
-            raise ValueError("'to_folder' %r must be a Folder instance" % to_folder)
-        return list(
-            i if isinstance(i, Exception) else Item.id_from_xml(i)
-            for i in self._consume_item_service(service_cls=MoveItem, items=ids, chunk_size=chunk_size, kwargs=dict(
-                to_folder=to_folder,
-            ))
-        )
+        return list(self._consume_item_service(service_cls=MoveItem, items=ids, chunk_size=chunk_size, kwargs=dict(
+            to_folder=to_folder,
+        )))
 
     def bulk_archive(self, ids, to_folder, chunk_size=None):
         """Archive items to a folder in the archive mailbox. An archive mailbox must be enabled in order for this
@@ -562,23 +527,39 @@ class Account(object):
 
         :param ids: an iterable of either (id, changekey) tuples or Item objects.
         :param to_folder: The destination folder of the archive operation
-        :param chunk_size: The number of items to send to the server in a single request
+        :param chunk_size: The number of items to send to the server in a single request (Default value = None)
+
         :return: A list containing True or an exception instance in stable order of the requested items
         """
-        if not isinstance(to_folder, (BaseFolder, FolderId, DistinguishedFolderId)):
-            raise ValueError("'to_folder' %r must be a Folder or FolderId instance" % to_folder)
         return list(self._consume_item_service(service_cls=ArchiveItem, items=ids, chunk_size=chunk_size, kwargs=dict(
                 to_folder=to_folder,
             ))
         )
 
-    def fetch(self, ids, folder=None, only_fields=None, chunk_size=None):
-        """ Fetch items by ID
+    def bulk_mark_as_junk(self, ids, is_junk, move_item, chunk_size=None):
+        """Mark or un-mark message items as junk email and add or remove the sender from the blocked sender list.
 
         :param ids: an iterable of either (id, changekey) tuples or Item objects.
-        :param folder: used for validating 'only_fields'
+        :param is_junk: Whether the messages are junk or not
+        :param move_item: Whether to move the messages to the junk folder or not
+        :param chunk_size: The number of items to send to the server in a single request (Default value = None)
+
+        :return: A list containing the new IDs of the moved items, if items were moved, or True, or an exception
+          instance, in stable order of the requested items.
+        """
+        return list(self._consume_item_service(service_cls=MarkAsJunk, items=ids, chunk_size=chunk_size, kwargs=dict(
+            is_junk=is_junk,
+            move_item=move_item,
+        )))
+
+    def fetch(self, ids, folder=None, only_fields=None, chunk_size=None):
+        """Fetch items by ID.
+
+        :param ids: an iterable of either (id, changekey) tuples or Item objects.
+        :param folder: used for validating 'only_fields' (Default value = None)
         :param only_fields: A list of string or FieldPath items specifying the fields to fetch. Default to all fields
-        :param chunk_size: The number of items to send to the server in a single request
+        :param chunk_size: The number of items to send to the server in a single request (Default value = None)
+
         :return: A generator of Item objects, in the same order as the input
         """
         validation_folder = folder or Folder(root=self.root)  # Default to a folder type that supports all item types
@@ -592,40 +573,54 @@ class Account(object):
             }
         else:
             for field in only_fields:
-                validation_folder.validate_item_field(field=field)
-            additional_fields = validation_folder.normalize_fields(fields=only_fields)
+                validation_folder.validate_item_field(field=field, version=self.version)
+            # Remove ItemId and ChangeKey. We get them unconditionally
+            additional_fields = {f for f in validation_folder.normalize_fields(fields=only_fields)
+                                 if not f.field.is_attribute}
         # Always use IdOnly here, because AllProperties doesn't actually get *all* properties
-        for i in self._consume_item_service(service_cls=GetItem, items=ids, chunk_size=chunk_size, kwargs=dict(
+        yield from self._consume_item_service(service_cls=GetItem, items=ids, chunk_size=chunk_size, kwargs=dict(
                 additional_fields=additional_fields,
                 shape=ID_ONLY,
-        )):
-            if isinstance(i, Exception):
-                yield i
-            else:
-                item = validation_folder.item_model_from_tag(i.tag).from_xml(elem=i, account=self)
-                yield item
+        ))
+
+    def fetch_personas(self, ids):
+        """Fetch personas by ID.
+
+        :param ids: an iterable of either (id, changekey) tuples or Persona objects.
+        :return: A generator of Persona objects, in the same order as the input
+        """
+        if isinstance(ids, QuerySet):
+            # We just want an iterator over the results
+            ids = iter(ids)
+        is_empty, ids = peek(ids)
+        if is_empty:
+            # We accept generators, so it's not always convenient for caller to know up-front if 'ids' is empty. Allow
+            # empty 'ids' and return early.
+            return
+        # GetPersona only accepts one persona ID per request. Crazy.
+        svc = GetPersona(account=self)
+        for i in ids:
+            yield svc.call(persona=i)
 
     @property
     def mail_tips(self):
-        """See self.oof_settings about caching considerations
-        """
+        """See self.oof_settings about caching considerations."""
         # mail_tips_requested must be one of properties.MAIL_TIPS_TYPES
-        res = list(GetMailTips(protocol=self.protocol).call(
+        return GetMailTips(protocol=self.protocol).get(
             sending_as=SendingAs(email_address=self.primary_smtp_address),
             recipients=[Mailbox(email_address=self.primary_smtp_address)],
             mail_tips_requested='All',
-        ))
-        if len(res) != 1:
-            raise ValueError('Expected result length 1, but got %s' % res)
-        if isinstance(res[0], Exception):
-            raise res[0]
-        return res[0]
+        )
 
     @property
     def delegates(self):
-        """Returns a list of DelegateUser objects representing the delegates that are set on this account
-        """
-        return list(GetDelegate(account=self).call(user_ids=None, include_permissions=True))
+        """Return a list of DelegateUser objects representing the delegates that are set on this account."""
+        delegates = []
+        for d in GetDelegate(account=self).call(user_ids=None, include_permissions=True):
+            if isinstance(d, Exception):
+                raise d
+            delegates.append(d)
+        return delegates
 
     def __str__(self):
         txt = '%s' % self.primary_smtp_address
