@@ -1,17 +1,13 @@
 import abc
-import base64
-import binascii
-from collections import OrderedDict
 import datetime
+import logging
+from collections import OrderedDict
 from decimal import Decimal, InvalidOperation
 from importlib import import_module
-import logging
 
-import dateutil.parser
-
-from .errors import ErrorInvalidServerVersion
 from .ewsdatetime import EWSDateTime, EWSDate, EWSTimeZone, NaiveDateTimeNotAllowed, UnknownTimeZone, UTC
-from .util import create_element, get_xml_attrs, set_xml_value, value_to_xml_text, is_iterable, safe_b64decode, TNS
+from .util import create_element, get_xml_attr, get_xml_attrs, set_xml_value, value_to_xml_text, is_iterable, \
+    xml_text_to_value, TNS
 from .version import Build, Version, EXCHANGE_2013
 
 log = logging.getLogger(__name__)
@@ -62,18 +58,27 @@ EXTRA_WEEKDAY_OPTIONS = (DAY, WEEK_DAY, WEEKEND_DAY)
 WEEKDAYS = WEEKDAY_NAMES + EXTRA_WEEKDAY_OPTIONS
 
 
+class InvalidField(ValueError):
+    """Used when a field name does not match any defined fields."""
+
+
+class InvalidFieldForVersion(ValueError):
+    """Used when a field is not supported on the given Exchnage version."""
+
+
+class InvalidChoiceForVersion(ValueError):
+    """Used when a value is not valid for an enum-type field."""
+
+
 def split_field_path(field_path):
-    """
+    """Split a string path into its field, label and subfield parts.
 
-    Args:
-      field_path:
+    :param field_path:
 
-    Returns:
-      Examples:
+    :return Examples:
       'start' -> ('start', None, None)
       'phone_numbers__PrimaryPhone' -> ('phone_numbers', 'PrimaryPhone', None)
       'physical_addresses__Home__street' -> ('physical_addresses', 'Home', 'street')
-
     """
     if not isinstance(field_path, str):
         raise ValueError("Field path %r must be a string" % field_path)
@@ -91,8 +96,9 @@ def split_field_path(field_path):
 
 
 def resolve_field_path(field_path, folder, strict=True):
-    # Takes the name of a field, or '__'-delimited path to a subfield, and returns the corresponding Field object,
-    # label and SubField object
+    """Take the name of a field, or '__'-delimited path to a subfield, and return the corresponding Field object,
+    label and SubField object.
+    """
     from .indexed_properties import SingleFieldIndexedElement, MultiFieldIndexedElement
     fieldname, label, subfieldname = split_field_path(field_path)
     field = folder.get_item_field_by_fieldname(fieldname)
@@ -151,7 +157,6 @@ class FieldPath:
     """Holds values needed to point to a single field. For indexed properties, we allow setting either field,
     field and label, or field, label and subfield. This allows pointing to either the full indexed property set, a
     property with a specific label, or a particular subfield field on that property.
-
     """
 
     def __init__(self, field, label=None, subfield=None):
@@ -195,8 +200,7 @@ class FieldPath:
             if not self.label or not self.subfield:
                 raise ValueError("Field path for indexed field '%s' is missing label and/or subfield" % self.field.name)
             return self.subfield.field_uri_xml(field_uri=self.field.field_uri, label=self.label)
-        else:
-            return self.field.field_uri_xml()
+        return self.field.field_uri_xml()
 
     def expand(self, version):
         # If this path does not point to a specific subfield on an indexed property, return all the possible path
@@ -234,7 +238,7 @@ class FieldPath:
 
 
 class FieldOrder:
-    """Holds values needed to call server-side sorting on a single field path"""
+    """Holds values needed to call server-side sorting on a single field path."""
 
     def __init__(self, field_path, reverse=False):
         if not isinstance(field_path, FieldPath):
@@ -258,7 +262,8 @@ class FieldOrder:
 
 
 class Field(metaclass=abc.ABCMeta):
-    """Holds information related to an item field"""
+    """Holds information related to an item field."""
+
     value_cls = None
     is_list = False
     # Is the field a complex EWS type? Quoting the EWS FindItem docs:
@@ -269,7 +274,7 @@ class Field(metaclass=abc.ABCMeta):
     #
     is_complex = False
 
-    def __init__(self, name, is_required=False, is_required_after_save=False, is_read_only=False,
+    def __init__(self, name=None, is_required=False, is_required_after_save=False, is_read_only=False,
                  is_read_only_after_send=False, is_searchable=True, is_attribute=False, default=None,
                  supported_from=None, deprecated_from=None):
         self.name = name
@@ -299,7 +304,7 @@ class Field(metaclass=abc.ABCMeta):
 
     def clean(self, value, version=None):
         if version and not self.supports_version(version):
-            raise ErrorInvalidServerVersion("Field '%s' does not support EWS builds prior to %s (server has %s)" % (
+            raise InvalidFieldForVersion("Field '%s' does not support EWS builds prior to %s (server has %s)" % (
                 self.name, self.supported_from, version))
         if value is None:
             if self.is_required and self.default is None:
@@ -322,11 +327,11 @@ class Field(metaclass=abc.ABCMeta):
 
     @abc.abstractmethod
     def from_xml(self, elem, account):
-        raise NotImplementedError()
+        pass
 
     @abc.abstractmethod
     def to_xml(self, value, version):
-        raise NotImplementedError()
+        pass
 
     def supports_version(self, version):
         # 'version' is a Version instance, for convenience by callers
@@ -343,7 +348,7 @@ class Field(metaclass=abc.ABCMeta):
 
     @abc.abstractmethod
     def __hash__(self):
-        raise NotImplementedError()
+        pass
 
     def __repr__(self):
         return self.__class__.__name__ + '(%s)' % ', '.join('%s=%r' % (f, getattr(self, f)) for f in (
@@ -351,6 +356,11 @@ class Field(metaclass=abc.ABCMeta):
 
 
 class FieldURIField(Field):
+    """A field that has a FieldURI value in EWS. This means it's value is contained in an XML element or arrtibute. It
+    may additionally be a label for searching, filtering and limiting fields. In that case, the FieldURI format will be
+    'itemtype:FieldName'
+    """
+
     def __init__(self, *args, **kwargs):
         self.field_uri = kwargs.pop('field_uri', None)
         self.namespace = kwargs.pop('namespace', TNS)
@@ -367,12 +377,18 @@ class FieldURIField(Field):
 
     def _get_val_from_elem(self, elem):
         if self.is_attribute:
-            return elem.get(self.field_uri)
-        field_elem = elem.find(self.response_tag())
-        return None if field_elem is None else field_elem.text or None
+            return elem.get(self.field_uri) or None
+        return get_xml_attr(elem, self.response_tag())
 
     def from_xml(self, elem, account):
-        raise NotImplementedError()
+        val = self._get_val_from_elem(elem)
+        if val is not None:
+            try:
+                return xml_text_to_value(val, self.value_cls)
+            except (ValueError, InvalidOperation):
+                log.warning("Cannot convert value '%s' on field '%s' to type %s", val, self.name, self.value_cls)
+                return None
+        return self.default
 
     def to_xml(self, value, version):
         field_elem = create_element(self.request_tag())
@@ -399,35 +415,18 @@ class FieldURIField(Field):
 
 
 class BooleanField(FieldURIField):
+    """A field that handles boolean values."""
+
     value_cls = bool
 
-    def __init__(self, *args, **kwargs):
-        self.true_val = kwargs.pop('true_val', 'true')
-        self.false_val = kwargs.pop('false_val', 'false')
-        super().__init__(*args, **kwargs)
-
-    def from_xml(self, elem, account):
-        val = self._get_val_from_elem(elem)
-        if val is not None:
-            try:
-                return {
-                    self.true_val: True,
-                    self.false_val: False,
-                }[val.lower()]
-            except KeyError:
-                log.warning("Cannot convert value '%s' on field '%s' to type %s", val, self.name, self.value_cls)
-                return None
-        return self.default
-
 
 class OnOffField(BooleanField):
-    def __init__(self, *args, **kwargs):
-        kwargs['true_val'] = 'on'
-        kwargs['false_val'] = 'off'
-        super().__init__(*args, **kwargs)
+    """A field that handles boolean values that are On/Off instead of True/False."""
 
 
 class IntegerField(FieldURIField):
+    """A field that handles integer values."""
+
     value_cls = int
 
     def __init__(self, *args, **kwargs):
@@ -435,42 +434,33 @@ class IntegerField(FieldURIField):
         self.max = kwargs.pop('max', None)
         super().__init__(*args, **kwargs)
 
+    def _clean_single_value(self, v):
+        if self.min is not None and v < self.min:
+            raise ValueError(
+                "Value %r on field '%s' must be greater than %s" % (v, self.name, self.min))
+        if self.max is not None and v > self.max:
+            raise ValueError("Value %r on field '%s' must be less than %s" % (v, self.name, self.max))
+
     def clean(self, value, version=None):
         value = super().clean(value, version=version)
         if value is not None:
             if self.is_list:
                 for v in value:
-                    if self.min is not None and v < self.min:
-                        raise ValueError(
-                            "Value %r on field '%s' must be greater than %s" % (v, self.name, self.min))
-                    if self.max is not None and v > self.max:
-                        raise ValueError("Value %r on field '%s' must be less than %s" % (v, self.name, self.max))
+                    self._clean_single_value(v)
             else:
-                if self.min is not None and value < self.min:
-                    raise ValueError("Value %r on field '%s' must be greater than %s" % (value, self.name, self.min))
-                if self.max is not None and value > self.max:
-                    raise ValueError("Value %r on field '%s' must be less than %s" % (value, self.name, self.max))
+                self._clean_single_value(value)
         return value
 
-    def from_xml(self, elem, account):
-        val = self._get_val_from_elem(elem)
-        if val is not None:
-            try:
-                return self.value_cls(val)
-            except (ValueError, InvalidOperation):
-                log.warning("Cannot convert value '%s' on field '%s' to type %s", val, self.name, self.value_cls)
-                return None
-        return self.default
-
 
 class DecimalField(IntegerField):
+    """A field that handles decimal values."""
+
     value_cls = Decimal
 
 
 class EnumField(IntegerField):
     """A field type where you can enter either the 1-based index in an enum (tuple), or the enum value. Values will be
     stored internally as integers but output in XML as strings.
-
     """
 
     def __init__(self, *args, **kwargs):
@@ -531,21 +521,16 @@ class EnumField(IntegerField):
 
 
 class EnumListField(EnumField):
+    """Like EnumField, but for lists of enum values."""
+
     is_list = True
 
 
 class EnumAsIntField(EnumField):
-    """Like EnumField, but communicates values with EWS in integers"""
+    """Like EnumField, but communicates values with EWS in integers."""
 
     def from_xml(self, elem, account):
-        val = self._get_val_from_elem(elem)
-        if val is not None:
-            try:
-                return int(val)
-            except ValueError:
-                log.warning("Cannot convert value '%s' on field '%s' to type %s", val, self.name, self.value_cls)
-                return None
-        return self.default
+        return super(EnumField, self).from_xml(elem=elem, account=account)
 
     def to_xml(self, value, version):
         field_elem = create_element(self.request_tag())
@@ -553,7 +538,8 @@ class EnumAsIntField(EnumField):
 
 
 class AppointmentStateField(IntegerField):
-    # MSDN: https://docs.microsoft.com/en-us/exchange/client-developer/web-service-reference/appointmentstate
+    """MSDN: https://docs.microsoft.com/en-us/exchange/client-developer/web-service-reference/appointmentstate"""
+
     NONE = 'None'
     MEETING = 'Meeting'
     RECEIVED = 'Received'
@@ -573,6 +559,8 @@ class AppointmentStateField(IntegerField):
 
 
 class Base64Field(FieldURIField):
+    """A field that handles binary data and automatically Base64 encodes and decodes the data."""
+
     value_cls = bytes
     is_complex = True
 
@@ -581,48 +569,31 @@ class Base64Field(FieldURIField):
             kwargs['is_searchable'] = False
         super().__init__(*args, **kwargs)
 
-    def from_xml(self, elem, account):
-        val = self._get_val_from_elem(elem)
-        if val is not None:
-            try:
-                return safe_b64decode(val)
-            except (TypeError, binascii.Error):
-                log.warning("Cannot convert value '%s' on field '%s' to type %s", val, self.name, self.value_cls)
-                return None
-        return self.default
-
-    def to_xml(self, value, version):
-        field_elem = create_element(self.request_tag())
-        return set_xml_value(field_elem, base64.b64encode(value).decode('ascii'), version=version)
-
 
 class MimeContentField(Base64Field):
-    # This element has an optional 'CharacterSet' attribute, but it specifies the encoding of the base64-encoded
-    # string (which doesn't make sense since base64 encoded strings are always ASCII). We ignore it here because
-    # the decoded data could be in some other encoding, specified in the "Content-Type:" header.
-    pass
+    """Like Base64Field. This element has an optional 'CharacterSet' attribute, but it specifies the encoding of the
+    base64-encoded string (which doesn't make sense since base64-encoded strings are always ASCII). We ignore it here
+    because the decoded data could be in some other encoding, specified in the "Content-Type" HTTP header.
+    """
 
 
 class DateField(FieldURIField):
+    """A field that handles date values."""
+
     value_cls = EWSDate
 
-    def from_xml(self, elem, account):
-        val = self._get_val_from_elem(elem)
-        if val is not None:
-            try:
-                return self.value_cls.from_string(val)
-            except ValueError:
-                log.warning("Cannot convert value '%s' on field '%s' to type %s", val, self.name, self.value_cls)
-                return None
-        return self.default
+    def clean(self, value, version=None):
+        # Allow plain datetime.date values as input
+        if type(value) is datetime.date:
+            value = self.value_cls.from_date(value)
+        return super().clean(value=value, version=version)
 
 
-class DateTimeBackedDateField(FieldURIField):
-    # A field that acts like a date, but where values are sent to EWS as EWSDateTime.
-    value_cls = datetime.date
+class DateTimeBackedDateField(DateField):
+    """A field that acts like a date, but where values are sent to EWS as EWSDateTime."""
 
     def __init__(self, *args, **kwargs):
-        # Not all fields assume a default tim of 00:00, so make this configurable
+        # Not all fields assume a default time of 00:00, so make this configurable
         self._default_time = kwargs.pop('default_time', datetime.time(0, 0))
         super().__init__(*args, **kwargs)
         # Create internal field to handle datetime-only logic
@@ -634,11 +605,11 @@ class DateTimeBackedDateField(FieldURIField):
     def from_xml(self, elem, account):
         val = self._get_val_from_elem(elem)
         if val is not None and len(val) == 25:
-            # This is a datetime string with timezone info. We don't want to have datetime values converted to UTC
-            # before converting to date. EWSDateTime.from_string() insists on converting to UTC, but we don't have an
-            # EWSTimeZone we can convert the timezone info to. Instead, parse the string manually when we have a
-            # datetime string with timezone info.
-            return EWSDate.from_date(dateutil.parser.parse(val).date())
+            # This is a datetime string with timezone info, e.g. '2021-03-01T21:55:54+00:00'. We don't want to have
+            # datetime values converted to UTC before converting to date. EWSDateTime.from_string() insists on
+            # converting to UTC, but we don't have an EWSTimeZone we can convert the timezone info to. Instead, parse
+            # the string with .fromisoformat().
+            return datetime.datetime.fromisoformat(val).date()
         # Revert to default parsing of datetime strings
         res = self._datetime_field.from_xml(elem=elem, account=account)
         if res is None:
@@ -652,6 +623,8 @@ class DateTimeBackedDateField(FieldURIField):
 
 
 class TimeField(FieldURIField):
+    """A field that handles time values."""
+
     value_cls = datetime.time
 
     def from_xml(self, elem, account):
@@ -661,20 +634,24 @@ class TimeField(FieldURIField):
                 if ':' in val:
                     # Assume a string of the form HH:MM:SS
                     return datetime.datetime.strptime(val, '%H:%M:%S').time()
-                else:
-                    # Assume an integer in minutes since midnight
-                    return (datetime.datetime(2000, 1, 1) + datetime.timedelta(minutes=int(val))).time()
+                # Assume an integer in minutes since midnight
+                return (datetime.datetime(2000, 1, 1) + datetime.timedelta(minutes=int(val))).time()
             except ValueError:
                 pass
         return self.default
 
 
 class DateTimeField(FieldURIField):
+    """A field that handles datetime values."""
+
     value_cls = EWSDateTime
 
     def clean(self, value, version=None):
-        if value is not None and isinstance(value, self.value_cls) and not value.tzinfo:
-            raise ValueError("Value '%s' on field '%s' must be timezone aware" % (value, self.name))
+        if isinstance(value, datetime.datetime):
+            if not value.tzinfo:
+                raise ValueError("Value '%s' on field '%s' must be timezone aware" % (value, self.name))
+            if type(value) is datetime.datetime:
+                value = self.value_cls.from_datetime(value)
         return super().clean(value, version=version)
 
     def from_xml(self, elem, account):
@@ -685,15 +662,14 @@ class DateTimeField(FieldURIField):
             except ValueError as e:
                 if isinstance(e, NaiveDateTimeNotAllowed):
                     # We encountered a naive datetime
-                    local_dt = e.args[0]
                     if account:
                         # Convert to timezone-aware datetime using the default timezone of the account
                         tz = account.default_timezone
-                        log.info('Found naive datetime %s on field %s. Assuming timezone %s', local_dt, self.name, tz)
-                        return local_dt.replace(tzinfo=tz)
+                        log.info('Found naive datetime %s on field %s. Assuming timezone %s', e.local_dt, self.name, tz)
+                        return e.local_dt.replace(tzinfo=tz)
                     # There's nothing we can do but return the naive date. It's better than assuming e.g. UTC.
-                    log.warning('Returning naive datetime %s on field %s', local_dt, self.name)
-                    return local_dt
+                    log.warning('Returning naive datetime %s on field %s', e.local_dt, self.name)
+                    return e.local_dt
                 log.info("Cannot convert value '%s' on field '%s' to type %s", val, self.name, self.value_cls)
                 return None
         return self.default
@@ -708,7 +684,6 @@ class DateOrDateTimeField(DateTimeField):
     For all-day calendar items, we assume both start and end dates are inclusive.
 
     For filtering kwarg validation and other places where we must decide on a specific class, we settle on datetime.
-
     """
 
     def __init__(self, *args, **kwargs):
@@ -719,7 +694,7 @@ class DateOrDateTimeField(DateTimeField):
     def clean(self, value, version=None):
         # Most calendar items will contain datetime values. We can't access the is_all_day value here, so CalendarItem
         # must handle that sanity check.
-        if type(value) == EWSDate:
+        if type(value) in (EWSDate, datetime.date):
             return self._date_field.clean(value=value, version=version)
         return super().clean(value=value, version=version)
 
@@ -732,8 +707,16 @@ class DateOrDateTimeField(DateTimeField):
 
 
 class TimeZoneField(FieldURIField):
+    """A field that handles timezone values."""
+
     value_cls = EWSTimeZone
 
+    def clean(self, value, version=None):
+        # Allow other timezone implementations as input
+        if value is not None:
+            value = self.value_cls.from_timezone(value)
+        return super().clean(value=value, version=version)
+
     def from_xml(self, elem, account):
         field_elem = elem.find(self.response_tag())
         if field_elem is not None:
@@ -757,18 +740,15 @@ class TimeZoneField(FieldURIField):
 
 
 class TextField(FieldURIField):
-    """A field that stores a string value with no length limit"""
+    """A field that stores a string value with no length limit."""
+
     value_cls = str
     is_complex = True
 
-    def from_xml(self, elem, account):
-        val = self._get_val_from_elem(elem)
-        if val is not None:
-            return val
-        return self.default
-
 
 class TextListField(TextField):
+    """Like TextField, but for lists of text."""
+
     is_list = True
 
     def from_xml(self, elem, account):
@@ -779,6 +759,8 @@ class TextListField(TextField):
 
 
 class MessageField(TextField):
+    """A field that handles the Message element."""
+
     INNER_ELEMENT_NAME = 'Message'
 
     def from_xml(self, elem, account):
@@ -798,7 +780,8 @@ class MessageField(TextField):
 
 
 class CharField(TextField):
-    """A field that stores a string value with a limited length"""
+    """A field that stores a string value with a limited length."""
+
     is_complex = False
 
     def __init__(self, *args, **kwargs):
@@ -825,7 +808,6 @@ class IdField(CharField):
     """A field to hold the 'Id' and 'Changekey' attributes on 'ItemId' type items. There is no guaranteed max length,
     but we can assume 512 bytes in practice. See
     https://docs.microsoft.com/en-us/exchange/client-developer/exchange-web-services/ews-identifiers-in-exchange
-
     """
 
     def __init__(self, *args, **kwargs):
@@ -836,10 +818,16 @@ class IdField(CharField):
 
 
 class CharListField(CharField):
+    """Like CharField, but for lists of strings."""
+
     is_list = True
 
+    def __init__(self, *args, **kwargs):
+        self.list_elem_name = kwargs.pop('list_elem_name', 'String')
+        super().__init__(*args, **kwargs)
+
     def list_elem_tag(self):
-        return '{%s}String' % self.namespace
+        return '{%s}%s' % (self.namespace, self.list_elem_name)
 
     def from_xml(self, elem, account):
         iter_elem = elem.find(self.response_tag())
@@ -849,25 +837,21 @@ class CharListField(CharField):
 
 
 class URIField(TextField):
-    """Helper to mark strings that must conform to xsd:anyURI
+    """Helper to mark strings that must conform to xsd:anyURI.
     If we want an URI validator, see http://stackoverflow.com/questions/14466585/is-this-regex-correct-for-xsdanyuri
-
     """
-    pass
 
 
 class EmailAddressField(CharField):
-    """A helper class used for email address string that we can use for email validation"""
-    pass
+    """A helper class used for email address string that we can use for email validation."""
 
 
 class CultureField(CharField):
     """Helper to mark strings that are # RFC 1766 culture values."""
-    pass
 
 
 class Choice:
-    """Implements versioned choices for the ChoiceField field"""
+    """Implement versioned choices for the ChoiceField field."""
 
     def __init__(self, value, supported_from=None):
         self.value = value
@@ -883,6 +867,8 @@ class Choice:
 
 
 class ChoiceField(CharField):
+    """Like CharField, but restricts the value to a limited set of strings."""
+
     def __init__(self, *args, **kwargs):
         self.choices = kwargs.pop('choices')
         super().__init__(*args, **kwargs)
@@ -891,13 +877,13 @@ class ChoiceField(CharField):
         value = super().clean(value, version=version)
         if value is None:
             return None
-        valid_choices = list(c.value for c in self.choices)
+        valid_choices = [c.value for c in self.choices]
         if version:
             valid_choices_for_version = self.supported_choices(version=version)
             if value in valid_choices_for_version:
                 return value
             if value in valid_choices:
-                raise ErrorInvalidServerVersion("Choice '%s' only supports EWS builds from %s to %s (server has %s)" % (
+                raise InvalidChoiceForVersion("Choice '%s' only supports EWS builds from %s to %s (server has %s)" % (
                     self.name, self.supported_from or '*', self.deprecated_from or '*', version))
         else:
             if value in valid_choices:
@@ -907,7 +893,7 @@ class ChoiceField(CharField):
         ))
 
     def supported_choices(self, version):
-        return list(c.value for c in self.choices if c.supports_version(version))
+        return [c.value for c in self.choices if c.supports_version(version)]
 
 
 FREE_BUSY_CHOICES = [Choice('Free'), Choice('Tentative'), Choice('Busy'), Choice('OOF'), Choice('NoData'),
@@ -915,12 +901,16 @@ FREE_BUSY_CHOICES = [Choice('Free'), Choice('Tentative'), Choice('Busy'), Choice
 
 
 class FreeBusyStatusField(ChoiceField):
+    """Like ChoiceField, but specifically for Free/Busy values."""
+
     def __init__(self, *args, **kwargs):
         kwargs['choices'] = set(FREE_BUSY_CHOICES)
         super().__init__(*args, **kwargs)
 
 
 class BodyField(TextField):
+    """A TextField with specific requirements for the Item body."""
+
     def __init__(self, *args, **kwargs):
         from .properties import Body
         self.value_cls = Body
@@ -955,6 +945,8 @@ class BodyField(TextField):
 
 
 class EWSElementField(FieldURIField):
+    """A generic field for any EWSElement object."""
+
     def __init__(self, *args, **kwargs):
         self._value_cls = kwargs.pop('value_cls')
         if 'namespace' not in kwargs:
@@ -992,6 +984,8 @@ class EWSElementField(FieldURIField):
 
 
 class EWSElementListField(EWSElementField):
+    """Like EWSElementField, but for lists of EWSElement objects."""
+
     is_list = True
     is_complex = True
 
@@ -1060,7 +1054,7 @@ class MessageHeaderField(EWSElementListField):
 
 
 class BaseEmailField(EWSElementField):
-    """A base class for EWSElement classes that have an 'email_address' field that we want to provide helpers for"""
+    """A base class for EWSElement classes that have an 'email_address' field that we want to provide helpers for."""
 
     is_complex = True  # FindItem only returns the name, not the email address
 
@@ -1127,8 +1121,8 @@ class MemberListField(EWSElementListField):
         super().__init__(*args, **kwargs)
 
     def clean(self, value, version=None):
+        from .properties import Mailbox
         if value is not None:
-            from .properties import Mailbox
             value = [
                 self.value_cls(mailbox=Mailbox(email_address=s)) if isinstance(s, str) else s for s in value
             ]
@@ -1150,6 +1144,8 @@ class AttendeesField(EWSElementListField):
 
 
 class AttachmentField(EWSElementListField):
+    """A field for item attachments."""
+
     def __init__(self, *args, **kwargs):
         from .attachments import Attachment
         kwargs['value_cls'] = Attachment
@@ -1170,7 +1166,7 @@ class AttachmentField(EWSElementListField):
 
 
 class LabelField(ChoiceField):
-    """A field to hold the label on an IndexedElement"""
+    """A field to hold the label on an IndexedElement."""
 
     def __init__(self, *args, **kwargs):
         super().__init__(*args, **kwargs)
@@ -1181,9 +1177,9 @@ class LabelField(ChoiceField):
 
 
 class SubField(Field):
-    namespace = TNS
+    """A field to hold the value on an SingleFieldIndexedElement."""
 
-    # A field to hold the value on an SingleFieldIndexedElement
+    namespace = TNS
     value_cls = str
 
     def from_xml(self, elem, account):
@@ -1202,7 +1198,8 @@ class SubField(Field):
 
 
 class EmailSubField(SubField):
-    """A field to hold the value on an SingleFieldIndexedElement"""
+    """A field to hold the value on an SingleFieldIndexedElement."""
+
     value_cls = str
 
     def from_xml(self, elem, account):
@@ -1210,7 +1207,8 @@ class EmailSubField(SubField):
 
 
 class NamedSubField(SubField):
-    """A field to hold the value on an MultiFieldIndexedElement"""
+    """A field to hold the value on an MultiFieldIndexedElement."""
+
     value_cls = str
 
     def __init__(self, *args, **kwargs):
@@ -1242,15 +1240,13 @@ class NamedSubField(SubField):
 
 
 class IndexedField(EWSElementField):
+    """A base class for all indexed fields."""
+
     PARENT_ELEMENT_NAME = None
 
     def to_xml(self, value, version):
         return set_xml_value(create_element('t:%s' % self.PARENT_ELEMENT_NAME), value, version)
 
-    def field_uri_xml(self):
-        # Callers must call field_uri_xml() on the subfield
-        raise NotImplementedError()
-
     def response_tag(self):
         return '{%s}%s' % (self.namespace, self.PARENT_ELEMENT_NAME)
 
@@ -1268,9 +1264,6 @@ class EmailAddressesField(IndexedField):
         kwargs['value_cls'] = EmailAddress
         super().__init__(*args, **kwargs)
 
-    def field_uri_xml(self):
-        raise NotImplementedError()
-
     def clean(self, value, version=None):
         if value is not None:
             default_labels = self.value_cls.LABEL_CHOICES
@@ -1296,9 +1289,6 @@ class PhoneNumberField(IndexedField):
         kwargs['value_cls'] = PhoneNumber
         super().__init__(*args, **kwargs)
 
-    def field_uri_xml(self):
-        raise NotImplementedError()
-
 
 class PhysicalAddressField(IndexedField):
     is_list = True
@@ -1310,9 +1300,6 @@ class PhysicalAddressField(IndexedField):
         kwargs['value_cls'] = PhysicalAddress
         super().__init__(*args, **kwargs)
 
-    def field_uri_xml(self):
-        raise NotImplementedError()
-
 
 class ExtendedPropertyField(Field):
     def __init__(self, *args, **kwargs):
@@ -1324,7 +1311,7 @@ class ExtendedPropertyField(Field):
             if self.is_required:
                 raise ValueError("'%s' is a required field" % self.name)
             return self.default
-        elif not isinstance(value, self.value_cls):
+        if not isinstance(value, self.value_cls):
             # Allow keeping ExtendedProperty field values as their simple Python type, but run clean() anyway
             tmp = self.value_cls(value)
             tmp.clean(version=version)
@@ -1368,7 +1355,6 @@ class ExtendedPropertyField(Field):
 class ItemField(FieldURIField):
     @property
     def value_cls(self):
-        # This is a workaround for circular imports. Item
         from .items import Item
         return Item
 
@@ -1408,12 +1394,11 @@ class EffectiveRightsField(EWSElementField):
 
 class BuildField(CharField):
     def __init__(self, *args, **kwargs):
-        from .version import Build
         super().__init__(*args, **kwargs)
         self.value_cls = Build
 
     def from_xml(self, elem, account):
-        val = super().from_xml(elem=elem, account=account)
+        val = self._get_val_from_elem(elem)
         if val:
             try:
                 return self.value_cls.from_hex_string(val)
@@ -1445,3 +1430,198 @@ class IdElementField(EWSElementField):
         kwargs['is_searchable'] = False
         kwargs['is_read_only'] = True
         super().__init__(*args, **kwargs)
+
+
+class TypeValueField(FieldURIField):
+    """This field type has no value_cls because values may have many different types."""
+
+    TYPES_MAP = {
+        'Boolean': bool,
+        'Integer32': int,
+        'UnsignedInteger32': int,
+        'Integer64': int,
+        'UnsignedInteger64': int,
+        # Python doesn't have a single-byte type to represent 'Byte'
+        'ByteArray': bytes,
+        'String': str,
+        'StringArray': str,  # A list of strings
+        'DateTime': EWSDateTime,
+    }
+    TYPES_MAP_REVERSED = {
+        bool: 'Boolean',
+        int: 'Integer64',
+        # Python doesn't have a single-byte type to represent 'Byte'
+        bytes: 'ByteArray',
+        str: 'String',
+        datetime.datetime: 'DateTime',
+        EWSDateTime: 'DateTime',
+    }
+
+    @classmethod
+    def get_type(cls, value):
+        if isinstance(value, bytes) and len(value) == 1:
+            # This is a single byte. Translate it to the 'Byte' type
+            return 'Byte'
+        if is_iterable(value):
+            # We don't allow generators as values, so keep the logic simple
+            try:
+                first = next(iter(value))
+            except StopIteration:
+                first = None
+            value_type = '%sArray' % cls.TYPES_MAP_REVERSED[type(first)]
+            if value_type not in cls.TYPES_MAP:
+                raise ValueError('%r is not a supported type' % value)
+            return value_type
+        return cls.TYPES_MAP_REVERSED[type(value)]
+
+    @classmethod
+    def is_array_type(cls, value_type):
+        return value_type == 'StringArray'
+
+    def clean(self, value, version=None):
+        if value is None:
+            if self.is_required and self.default is None:
+                raise ValueError("'%s' is a required field with no default" % self.name)
+            return self.default
+        return value
+
+    def from_xml(self, elem, account):
+        field_elem = elem.find(self.response_tag())
+        if field_elem is None:
+            return self.default
+        value_type_str = get_xml_attr(field_elem, '{%s}Type' % TNS)
+        value = get_xml_attr(field_elem, '{%s}Value' % TNS)
+        if value_type_str == 'Byte':
+            try:
+                # The value is an unsigned integer in the range 0 -> 255. Convert it to a single byte
+                return xml_text_to_value(value, int).to_bytes(1, 'little', signed=False)
+            except OverflowError as e:
+                log.warning('Invalid byte value %r (%e)', value, e)
+                return None
+        value_type = self.TYPES_MAP[value_type_str]
+        if self. is_array_type(value_type_str):
+            return tuple(xml_text_to_value(value=v, value_type=value_type) for v in value.split(' '))
+        return xml_text_to_value(value=value, value_type=value_type)
+
+    def to_xml(self, value, version):
+        value_type_str = self.get_type(value)
+        if value_type_str == 'Byte':
+            # A single byte is encoded to an unsigned integer in the range 0 -> 255
+            value = int.from_bytes(value, byteorder='little', signed=False)
+        elif is_iterable(value):
+            value = ' '.join(value_to_xml_text(v) for v in value)
+        field_elem = create_element(self.request_tag())
+        field_elem.append(set_xml_value(create_element('t:Type'), value_type_str, version=version))
+        field_elem.append(set_xml_value(create_element('t:Value'), value, version=version))
+        return field_elem
+
+
+class DictionaryField(FieldURIField):
+    value_cls = dict
+
+    def from_xml(self, elem, account):
+        from .properties import DictionaryEntry
+        iter_elem = elem.find(self.response_tag())
+        if iter_elem is not None:
+            entries = [
+                DictionaryEntry.from_xml(elem=e, account=account)
+                for e in iter_elem.findall(DictionaryEntry.response_tag())
+            ]
+            return {e.key: e.value for e in entries}
+        return self.default
+
+    def clean(self, value, version=None):
+        if isinstance(value, dict):
+            cleaned = {}
+            for k, v in value.items():
+                if type(k) is datetime.datetime:
+                    k = EWSDateTime.from_datetime(k)
+                if type(v) is datetime.datetime:
+                    v = EWSDateTime.from_datetime(v)
+                cleaned[k] = v
+            value = cleaned
+        return super().clean(value=value, version=version)
+
+    def to_xml(self, value, version):
+        from .properties import DictionaryEntry
+        field_elem = create_element(self.request_tag())
+        entries = [DictionaryEntry(key=k, value=v) for k, v in value.items()]
+        return set_xml_value(field_elem, entries, version=version)
+
+
+class PersonaPhoneNumberField(EWSElementField):
+    is_complex = True
+
+    def __init__(self, *args, **kwargs):
+        from .properties import PhoneNumber
+        kwargs['value_cls'] = PhoneNumber
+        super().__init__(*args, **kwargs)
+
+
+class BodyContentAttributedValueField(EWSElementField):
+    is_complex = True
+
+    def __init__(self, *args, **kwargs):
+        from .properties import BodyContentAttributedValue
+        kwargs['value_cls'] = BodyContentAttributedValue
+        super().__init__(*args, **kwargs)
+
+
+class StringAttributedValueField(EWSElementListField):
+    def __init__(self, *args, **kwargs):
+        from .properties import StringAttributedValue
+        kwargs['value_cls'] = StringAttributedValue
+        super().__init__(*args, **kwargs)
+
+
+class PhoneNumberAttributedValueField(EWSElementListField):
+    def __init__(self, *args, **kwargs):
+        from .properties import PhoneNumberAttributedValue
+        kwargs['value_cls'] = PhoneNumberAttributedValue
+        super().__init__(*args, **kwargs)
+
+
+class EmailAddressAttributedValueField(EWSElementListField):
+    def __init__(self, *args, **kwargs):
+        from .properties import EmailAddressAttributedValue
+        kwargs['value_cls'] = EmailAddressAttributedValue
+        super().__init__(*args, **kwargs)
+
+
+class PostalAddressAttributedValueField(EWSElementListField):
+    def __init__(self, *args, **kwargs):
+        from .properties import PostalAddressAttributedValue
+        kwargs['value_cls'] = PostalAddressAttributedValue
+        super().__init__(*args, **kwargs)
+
+
+class GenericEventListField(EWSElementField):
+    """A list field that can contain all subclasses of Event."""
+
+    is_list = True
+
+    @property
+    def _event_types_map(self):
+        return {v.response_tag(): v for v in self.value_classes}
+
+    def __init__(self, *args, **kwargs):
+        from .properties import CopiedEvent, CreatedEvent, DeletedEvent, ModifiedEvent, MovedEvent, \
+            NewMailEvent, StatusEvent, FreeBusyChangedEvent
+        kwargs['value_cls'] = None  # Parent class requires this kwarg
+        kwargs['namespace'] = None  # Parent class requires this kwarg
+        super().__init__(*args, **kwargs)
+        self.value_classes = (
+            CopiedEvent, CreatedEvent, DeletedEvent, ModifiedEvent, MovedEvent, NewMailEvent, StatusEvent,
+            FreeBusyChangedEvent,
+        )
+
+    def from_xml(self, elem, account):
+        events = []
+        for event in elem:
+            # This may or may not be an event element. Could also be other child elements of Notification
+            try:
+                value_cls = self._event_types_map[event.tag]
+            except KeyError:
+                continue
+            events.append(value_cls.from_xml(elem=event, account=account))
+        return events or self.default
