@@ -119,7 +119,7 @@ class UID(bytes):
         property_type = 'Binary'
 
     CalendarItem.register('global_object_id', GlobalObjectId)
-    account.calendar.filter(global_object_id=GlobalObjectId(UID('261cbc18-1f65-5a0a-bd11-23b1e224cc2f')))
+    account.calendar.filter(global_object_id=UID('261cbc18-1f65-5a0a-bd11-23b1e224cc2f'))
     """
 
     _HEADER = binascii.hexlify(bytearray((
@@ -152,6 +152,11 @@ class UID(bytes):
         ])
         return super().__new__(cls, codecs.decode(encoding, 'hex'))
 
+    @classmethod
+    def to_global_object_id(cls, uid):
+        """Converts a UID as returned by EWS to GlobalObjectId format"""
+        return binascii.unhexlify(uid)
+
 
 def _mangle(field_name):
     return '__%s' % field_name
