@@ -5,7 +5,7 @@ from ..version import EXCHANGE_2010
 
 
 class GetRoomLists(EWSService):
-    """MSDN: https://docs.microsoft.com/en-us/exchange/client-developer/web-service-reference/getroomlists"""
+    """MSDN: https://docs.microsoft.com/en-us/exchange/client-developer/web-service-reference/getroomlists-operation"""
 
     SERVICE_NAME = 'GetRoomLists'
     element_container_name = '{%s}RoomLists' % MNS
