@@ -1,19 +1,19 @@
-from ..util import create_element, set_xml_value, MNS
 from .common import EWSService
+from ..properties import FreeBusyView
+from ..util import create_element, set_xml_value, MNS
 
 
 class GetUserAvailability(EWSService):
-    """Get detailed availability information for a list of users
+    """Get detailed availability information for a list of users.
     MSDN:
     https://docs.microsoft.com/en-us/exchange/client-developer/web-service-reference/getuseravailability-operation
-
     """
+
     SERVICE_NAME = 'GetUserAvailability'
 
     def call(self, timezone, mailbox_data, free_busy_view_options):
         # TODO: Also supports SuggestionsViewOptions, see
-        # https://docs.microsoft.com/en-us/exchange/client-developer/web-service-reference/suggestionsviewoptions
-        from ..properties import FreeBusyView
+        #  https://docs.microsoft.com/en-us/exchange/client-developer/web-service-reference/suggestionsviewoptions
         for elem in self._get_elements(payload=self.get_payload(
             timezone=timezone,
             mailbox_data=mailbox_data,
@@ -45,8 +45,8 @@ class GetUserAvailability(EWSService):
         for msg in response:
             # Just check the response code and raise errors
             self._get_element_container(message=msg.find('{%s}ResponseMessage' % MNS))
-            for c in self._get_elements_in_container(container=msg):
-                yield c
+            yield from self._get_elements_in_container(container=msg)
 
-    def _get_elements_in_container(self, container):
+    @classmethod
+    def _get_elements_in_container(cls, container):
         return [container.find('{%s}FreeBusyView' % MNS)]
