@@ -1,20 +1,23 @@
-from ..util import create_element, set_xml_value, MNS
 from .common import EWSAccountService, create_item_ids_element
+from ..util import create_element, set_xml_value, MNS
 
 
 class MoveItem(EWSAccountService):
     """MSDN: https://docs.microsoft.com/en-us/exchange/client-developer/web-service-reference/moveitem-operation"""
+
     SERVICE_NAME = 'MoveItem'
     element_container_name = '{%s}Items' % MNS
 
     def call(self, items, to_folder):
-        from ..folders import BaseFolder, FolderId, DistinguishedFolderId
-        if not isinstance(to_folder, (BaseFolder, FolderId, DistinguishedFolderId)):
+        from ..folders import BaseFolder, FolderId
+        from ..items import Item
+        if not isinstance(to_folder, (BaseFolder, FolderId)):
             raise ValueError("'to_folder' %r must be a Folder or FolderId instance" % to_folder)
-        return self._get_elements(payload=self.get_payload(
-            items=items,
-            to_folder=to_folder,
-        ))
+        for elem in self._chunked_get_elements(self.get_payload, items=items, to_folder=to_folder):
+            if isinstance(elem, (Exception, type(None))):
+                yield elem
+                continue
+            yield Item.id_from_xml(elem)
 
     def get_payload(self, items, to_folder):
         # Takes a list of items and returns their new item IDs
